/*
!C******************************************************************

!Description: 
		ancillary.c


!Input Parameters:
!Output Parameters: 
!Revision History:
!Team-unique Header:
!References and Credits 
!Design Notes


DESCRIPTION


!END*************************************************************************/

/* NODAAC may need to be defined when running this code
in any non-DAAC processing mode.  This feature was added
by MODIS SDST to ensure compliance with ESDIS standards upon
delivering this software to the DAAC. */

#ifdef NODAAC

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "PGS_MODIS_37092.h"

#include "hdf.h"               /* HDF header files */
#include "mfhdf.h"

#include "meta.h"
#include "protos.h"
#ifdef DEBUG_MALLOC
#include "/usr/include/dmalloc.h"
#endif

#define UNDEFINED 9.999e20     /* Has to be same as in grib.h */
static float *tmpfltarray[7][2] = {{NULL,NULL},{NULL,NULL},{NULL,NULL},
                                   {NULL,NULL},{NULL,NULL},{NULL,NULL},{NULL,NULL}};
extern d_ancillary daoozone;   /* this will refer to EP-TOMS ozone */
extern d_ancillary gdasozone;  /* this will refer to GDAS ozone, the EP-TOMS backup */ 
extern d_ancillary daospres;
extern d_ancillary daowv;
extern d_ancillary daotmp;
extern d_ancillary daouwnd;
extern d_ancillary daovwnd;
extern d_ancillary avg_z; 

PGSt_SMF_status read_dao_plane(PGSt_PC_Logical file_lun, 
			       char *rbdt, char *daovar, d_ancillary *ancillary)
/*
!C*****************************************************************************
!Description: On the basis of time, decides which plane of data in
              a DAO file should be read, and reads in that plane of data.

!Input Parameters:
file_lun: DAO File logical unit number
version: version number of the file to be read
rdbt: beginning time
daovar: name of the SDS to be read from the file
ancillary: d_ancillary ancillary data structure

!Output Parameters:
data: pointer to hold the data read
!Input/Output Parameters: none

!Returns: on success: MODIS_S_SUCCESS 
          on failure: MODIS_E_HDF_FAILURE
                      MODIS_E_GETREF_FAIL
!Revision History:
        Revision 2.0, P. Fisher, SSAI/NASA's GSFC Code 923
	1.  Modified to accurately read in DAO files beginning at 21:00

        Revision 2.0, P. Fisher, SSAI/NASA's GSFC Code 923
	1.  Modified, to select the correct data plane out of two potential 
	input files (days).
	2.  Data buffered, so that the same ancillary data is not read 
        in twice in a row.

        Revision 1.1, P. Fisher, SSAI/NASA's GSFC Code 923
	1. Revised to accept the d_ancillary structure as input
	2. Revised to read the line and sample parameters directly
	   from the input file.

	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes
1.  This function assumes the DAO files are 24 hours, beginning at 21:00 and
    divided into 3 hour time slices.
!END***************************************************************************
*/
{
  int32 sdid;
  int32 index;
  int32 sdsid;
  int32 hour;
  char chour[PGSd_PC_VALUE_LENGTH_MAX];
  int32 timeplane;
  float32 dectime;
  int32 start[3]={0,0,0};
  int32 edge[3]={1,1};
  char sds_name[200];
  int32 rank;
  int32 data_type;
  int32 nattrs;
  int32 timestep;
  float32 *datap;
  char hour_st[3], *tp;

  PGSt_double julian[2];
  int jules;

  char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */
  char tf_name[PGSd_PC_FILE_PATH_MAX]={'\0'};	    
  int32 i;
  PGSt_integer ver, sver;
  char location[100]="ancillary.c, read_dao_plane\0";

  if(PGS_TD_UTCtoUTCjd(rbdt, julian)!=PGS_S_SUCCESS){
    modsmf(MODIS_E_BAD_TIME_CONV, "Unable to convert date/time of granule to julian date.  "
	   "Unable to process DAO data.", location);
    return(MODIS_E_BAD_TIME_CONV);
  }
  jules=(int)julian[0];

  /* note that dao files start @ 2100 the day before -- usually, but not always */
  tp=strchr(rbdt, 'T');
  tp++;

  strncpy(hour_st, tp, 2);
  hour_st[2]='\0';
  hour=atoi(hour_st);
 
  i = 0;
  
  sver=ver=(PGSt_integer)ancillary->versions[i];
  if(PGS_PC_GetReference(file_lun, &ver, tf_name)!=PGS_S_SUCCESS){
    sprintf(msg, "LUN %d, version %d", (int)file_lun, (int)ancillary->versions[i]);
    modsmf((PGSt_SMF_code)MODIS_E_GETREF_FAIL, msg, location);
    return(MODIS_E_GETREF_FAIL);
  }

  
  if((sdid=SDstart(tf_name, DFACC_RDONLY))==FAIL) {
    sprintf(msg, "LUN %d, version %d", (int32)file_lun, (int)ancillary->versions[i]);    
    modsmf(MODIS_E_OPEN_HDF_FILE, msg, location);
    return (MODIS_E_OPEN_HDF_FILE);
  }

  if((index=SDnametoindex(sdid, daovar))==FAIL){ 
    modsmf(MODIS_E_HDF_FAILURE, "SDnametoindex()", location);
    SDend(sdid);
    return (MODIS_E_HDF_FAILURE);
  }

  if((sdsid=SDselect(sdid, index))==FAIL){
    modsmf(MODIS_E_HDF_FAILURE, "SDselect()",  location);
    SDend(sdid);
    return (MODIS_E_HDF_FAILURE);
  }
    
  if(SDgetinfo(sdsid, sds_name, &rank, ancillary->dims, &data_type, &nattrs)==FAIL){
    modsmf(MODIS_E_HDF_FAILURE, "SDgetinfo()", location);
    SDendaccess(sdsid);
    SDend(sdid);
    return (MODIS_E_HDF_FAILURE);
  }

  if(SDgetfillvalue(sdsid, &(ancillary->fillvalue))==FAIL){
       modsmf(MODIS_E_HDF_FAILURE, "SDgetfillvalue(), using default DAO fill value", 
	      location);
       ancillary->fillvalue=9.9999999e+14F;
  }

  /* get latitude and longitude resolution */
  ancillary->lonres=360.0/ancillary->dims[2];
  ancillary->latres=180.0/(ancillary->dims[1]-1);
    
  
  timestep=24/ancillary->dims[0];
  /* Assume three hour slices */
  if(timestep!=3){
    sprintf(msg, "LUN %d.  There are more or less than 8 time slices in the file", (int)file_lun);
    modsmf(MODIS_E_DAO_FAIL, msg, location);
  }
  
  /* Which plane do we need... */
  timeplane=hour/timestep;
  
  /* shift is necessary IF the file starts at 2100 hours */
  if (ancillary->jrbd[i] == 21) timeplane++;
  
  if (timeplane>=ancillary->dims[0])timeplane=0;
    
  /* don't read again, if the correct data is already in the buffer */
  if(timeplane!=ancillary->lasttimeplane || sver!=ancillary->lastversion[0]){
    
    
    ancillary->lasttimeplane=timeplane;
    ancillary->lastversion[0]=sver;
    
    
    /* allocate data array -- for safety's sake, all DAO files may not have the same 
       dimensions -- who knows!!!*/
    ancillary->data=(float32 *)ReAllocMem((void *)ancillary->data, 
					  ancillary->dims[1]*ancillary->dims[2]*sizeof(float32));
    
    /* initialize and transfer to maintain portability */
    start[0]=(int32)timeplane;
    edge[2]=(int32)ancillary->dims[2];
    
    datap=ancillary->data;
    
    for(i=ancillary->dims[1]-1;i>-1;i--){
      
      start[1]=(int32)i;

      if(SDreaddata(sdsid, start, NULL, edge, (VOID *)datap)==FAIL){
	modsmf(MODIS_E_HDF_FAILURE, "SDreaddata()",  location);
	SDendaccess(sdsid);
	SDend(sdid);
	return (MODIS_E_HDF_FAILURE);
      }
      
      datap+=ancillary->dims[2];
    }
  }/* if(timeplane... */
  
  SDendaccess(sdsid);
  SDend(sdid);
  
  
  return(MODIS_S_SUCCESS);
  
}


PGSt_SMF_status read_plane(char *rdbt, d_ancillary *ancillary)
/*
!C*****************************************************************************
!Description: On the basis of preset tags in the ancillary structure, tries
              to read DAO or GRIB data.

!Input Parameters:
rdbt:      time
ancillary: d_ancillary ancillary data structure

!Output Parameters:
none

!Input/Output Parameters: 
ancillary is updated.

!Returns: on success: MODIS_S_SUCCESS 
          on failure: MODIS_E_HDF_FAILURE
                      MODIS_E_GETREF_FAIL
!Revision History:

	Original version 1999, Jim Ray (jim@cratmos.gsfc.nasa.gov), derived
	                       from read_dao_plane() (above).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Jim Ray
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		jim@cratmos.gsfc.nasa.gov, (301) 614-6684

!Design Notes

!END***************************************************************************
*/
{
  int32 timeplane, hour;
  char hour_st[3], *tp;
  PGSt_double julian[2];
  int jules;
  
  /* variables for reading from GRIB files */
  FILE *fd;
  char where[15];
  int nx,ny;
  int grib_ret;
  int will_read, will_read_interpolate;
  int tempfltindex;
  int read_grib_array(FILE *input, char *what, char *where, int *nx, int *ny, float **narray);

  char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */
  char tf_name[PGSd_PC_FILE_PATH_MAX]={'\0'};	    
  int32 i,j;
  PGSt_integer ver, sver;
  PGSt_integer index;
  char location[100]="ancillary.c, read_plane\0";
  /* figure out which file to read */
  if (!strcmp(ancillary->anci_source, "DAO\0")) {
  
     /* If it's DAO, get the right LUN and call read_dao_plane() */
     if (!strcmp(ancillary->tag, DAOOZONEVARIABLE)) index = DAOOZONE;
     else if (!strcmp(ancillary->tag, DAOWVVARIABLE)) index = DAOWV;
     else if (!strcmp(ancillary->tag, DAOSPRESVARIABLE)) index = DAOSPRES;
     else {
        sprintf(msg, "Unexpected ancillary data file tag: '%s'", ancillary->tag);
        modsmf(MODIS_F_CODING_ERROR, msg, location);
     }
     return (read_dao_plane(index, rdbt, ancillary->tag, ancillary));

                                                } /* if (!strcmp(ancillary->anci_source, "DA.. */
						
  else if ( !strcmp(ancillary->anci_source, "GRIB\0")   || !strcmp(ancillary->anci_source, "EPT\0")) {

     /* If it's GRIB, get the right LUN */
     if (!strcmp(ancillary->tag, OZONE_GRIBTAG)) {
        index = EPT_OZ;
        strcpy(where, OZONE_GRIBLEVEL);
	tempfltindex=0;
	}
     else if (!strcmp(ancillary->tag, GOZONE_GRIBTAG)) {
        index = GRIB_WV;     /* get from GDAS file */
        strcpy(where, OZONE_GRIBLEVEL);
	tempfltindex=1;
	}
     else if (!strcmp(ancillary->tag, WV_GRIBTAG)) {
        index = GRIB_WV;
        strcpy(where, WV_GRIBLEVEL);
	tempfltindex=2;
	}
     else if (!strcmp(ancillary->tag, SPRES_GRIBTAG)) {
        index = GRIB_SP;
        strcpy(where, SPRES_GRIBLEVEL);
	tempfltindex=3;
	}
     else if (!strcmp(ancillary->tag, TMP_GRIBTAG)) { /* 01-MAY-01 */
        index = GRIB_SP;  /* read from the same file you read SP from */
        strcpy(where, TMP_GRIBLEVEL);
	tempfltindex=4;
	}
     else if (!strcmp(ancillary->tag, UWD_GRIBTAG)) { /* 14-AUG-01 */
        index = GRIB_SP;  /* read from the same file you read SP from */
        strcpy(where, UWD_GRIBLEVEL);
	tempfltindex=5;
	}
     else if (!strcmp(ancillary->tag, VWD_GRIBTAG)) { /* 14-AUG-01 */
        index = GRIB_SP;  /* read from the same file you read SP from */
        strcpy(where, VWD_GRIBLEVEL);
	tempfltindex=6;
	}
     else {
        sprintf(msg, "Unexpected ancillary data file tag: '%s'", ancillary->tag);
        modsmf(MODIS_F_CODING_ERROR, msg, location);
    }
     
     will_read=will_read_interpolate=0;
     
     if (ancillary->numdays==0) {
         /* then you don't have to read two sets of data, just one... */
	 
	 if (ancillary->versions[0] != ancillary->lastversion[0]){  /* if it hasn't been read yet */
	 
	    will_read = 1;
            sver=ver=(PGSt_integer)ancillary->versions[0];
            if(PGS_PC_GetReference(index, &ver, tf_name)!=PGS_S_SUCCESS){
               sprintf(msg, "LUN %d, version %d", (int)index, (int)ancillary->versions[0]);
               modsmf((PGSt_SMF_code)MODIS_E_GETREF_FAIL, msg, location);
               return(MODIS_E_GETREF_FAIL);
            }
            nx = ny = 0;
            if ((fd=fopen(tf_name,"rb")) == NULL) {
               sprintf(msg, "LUN %d, version %d", (int32)index, sver);
               modsmf(MODIS_E_OPEN_GEN_FILE, msg, location);
               return(MODIS_E_OPEN_GEN_FILE);
   	    }
	    /* 11-OCT-04: need to read into the tmpfltarray and *then* transfer to the ancillary->data array
	       (below), even if we are not weighting the data, so that ancillary->data array is refreshed 
	       even if data is not being read -- or else we wind up scaling the data twice (once for each pass). */
	    grib_ret=read_grib_array(fd, ancillary->tag, where, &ny, &nx, &tmpfltarray[tempfltindex][0]);
            /*printf("%s\n*** return value = %d, read %s, %d by %d *** \n", 
	        tf_name,grib_ret,ancillary->tag, nx, ny); */
             
	    fclose(fd);
            ancillary->lastversion[0]=sver;
	   	  
            /* for now, use MODIS_E_HDF_FAILURE */
            if (grib_ret) {
	       return (MODIS_E_HDF_FAILURE);
	       }
	 
            ancillary->dims[2] = (int32)nx;
            ancillary->dims[1] = (int32)ny;
            ancillary->dims[0] = (int32)1;  /* ?? */
 
            /* get latitude and longitude resolution */
            ancillary->lonres=360.0/ancillary->dims[2];
            ancillary->latres=180.0/(ancillary->dims[1]-1);
       
            /* Note: as far as I know, there are no fill values in GRIB files. 
	       Routine 'read_grib_array()' uses UNDEFINED for missing values;
	       use same thing for fill values (14-JUL-99, Jim Ray, SSAI).  */
            ancillary->fillvalue=UNDEFINED;
	 
	 }
	 
         ancillary->data = (float *) ReAllocMem(ancillary->data, ancillary->dims[2]*ancillary->dims[1]*sizeof(float));	 
	 for (j=0;j<ancillary->dims[2]*ancillary->dims[1];j++) {
	     ancillary->data[j] = tmpfltarray[tempfltindex][0][j];
	     }
	 
	 
     }  /* if (ancillary->numdays==0) */
     else {
	 if ((ancillary->versions[0] != ancillary->lastversion[0])||
             (ancillary->versions[1] != ancillary->lastversion[1])) {  /* if they haven't been read yet */

	     will_read_interpolate = 1;	     
	     for(i=0;i<=ancillary->numdays;i++) {  /* 03-MAY-01 */

                 sver=ver=(PGSt_integer)ancillary->versions[i];
                 if(PGS_PC_GetReference(index, &ver, tf_name)!=PGS_S_SUCCESS){
                    sprintf(msg, "LUN %d, version %d", (int)index, (int)ancillary->versions[i]);
                    modsmf((PGSt_SMF_code)MODIS_E_GETREF_FAIL, msg, location);
                    return(MODIS_E_GETREF_FAIL);
                 }
                 nx = ny = 0;
                 if ((fd=fopen(tf_name,"rb")) == NULL) {
                    sprintf(msg, "LUN %d, version %d", (int32)index, sver);
                    modsmf(MODIS_E_OPEN_GEN_FILE, msg, location);
                    return(MODIS_E_OPEN_GEN_FILE);
   	         }
		 
 	         if (tempfltindex == 0)   /* 29-APR-04: you are reading OZONE, use EP-TOMS reader */
	             grib_ret=read_EPT_array(fd, &ny, &nx, &tmpfltarray[tempfltindex][i]);
		 else   
 	             grib_ret=read_grib_array(fd, ancillary->tag, where, &ny, &nx, &tmpfltarray[tempfltindex][i]);
/*
 *printf("%s return value = %d, read %s, %d by %d tempflindex = %d***\n\n", 
 *	           location,grib_ret,ancillary->tag, nx, ny, tempfltindex);
 */                
	         fclose(fd);
                 ancillary->lastversion[i]=sver;
		 
                 /* for now, use MODIS_E_HDF_FAILURE */
                 if (grib_ret) {
	            return (MODIS_E_HDF_FAILURE);
	            }
	 
                 ancillary->dims[2] = (int32)nx;
                 ancillary->dims[1] = (int32)ny;
                 ancillary->dims[0] = (int32)1;  /* ?? */
 
                 /* get latitude and longitude resolution */
                 ancillary->lonres=360.0/ancillary->dims[2];
                 ancillary->latres=180.0/(ancillary->dims[1]-1);
       
                 /* Note: as far as I know, there are no fill values in GRIB files. 
	            Routine 'read_grib_array()' uses UNDEFINED for missing values;
	            use same thing for fill values (14-JUL-99, Jim Ray, SSAI).  */
                 ancillary->fillvalue=UNDEFINED;
		 
	     }
	 }
	 
	 /* Apply weights to the data; freshly read or not */
         ancillary->data = (float *) ReAllocMem(ancillary->data, ancillary->dims[2]*ancillary->dims[1]*sizeof(float));	 

         if (tempfltindex == 0) {  /* EP-TOMS ozone */
	     /* 30-APR-04: Set the higher weight to 1.0, the lower to 0.0 */
	     if (ancillary->weights[0] > ancillary->weights[1]) {
	         ancillary->weights[0] = 1.0F;
	         ancillary->weights[1] = 0.0F;
	        }
	     else {
	         ancillary->weights[0] = 0.0F;
	         ancillary->weights[1] = 1.0F;
	        }
	     }
/*
 * printf("%s: Applying weights %f (to %f) and %f (to %f), %d\n", ancillary->tag, ancillary->weights[0], tmpfltarray[tempfltindex][0][0], ancillary->weights[1], tmpfltarray[tempfltindex][1][0], tempfltindex);
 */
	 for (j=0;j<ancillary->dims[2]*ancillary->dims[1];j++) {
	     ancillary->data[j] = tmpfltarray[tempfltindex][0][j]*ancillary->weights[0] +
	                          tmpfltarray[tempfltindex][1][j]*ancillary->weights[1];
	     }
	     
         /*if (tempfltindex==2){
	 {FILE *fd;
	 char filename[50];
	 struct timeval  first;
         struct timezone tzp;

         gettimeofday (&first, &tzp);
 	 sprintf(filename, "array_%d_0.img\0", first.tv_usec);
	 fd=fopen(filename, "wb");
	 fwrite(tmpfltarray[tempfltindex][0], ancillary->dims[2]*ancillary->dims[1]*sizeof(float),1,fd);
	 fclose(fd);

	 sprintf(filename, "array_%d_1.img\0", first.tv_usec);
	 fd=fopen(filename, "wb");
	 fwrite(tmpfltarray[tempfltindex][1], ancillary->dims[2]*ancillary->dims[1]*sizeof(float),1,fd);
	 fclose(fd);


	 sprintf(filename, "array_%d_%f_%f.img\0", first.tv_usec, ancillary->weights[0],
	               ancillary->weights[1]);
	 fd=fopen(filename, "wb");
	 fwrite(ancillary->data, ancillary->dims[2]*ancillary->dims[1]*sizeof(float),1,fd);
	 fclose(fd);
	 }
	 }*/
     }
     
     /* 26-APR-01: convert NCEP units into our units: 
	PWAT is in kg/m^2, convert to g/cm^2 (divide by 10.0)
	SPRES is in Pa, convert to mbar (divide by 100.0).
      */
     if (!strcmp(ancillary->tag, SPRES_GRIBTAG)) {
	   for (j=0;j<ancillary->dims[2]*ancillary->dims[1];j++) ancillary->data[j] /= 100.0F;
        }
     else if (!strcmp(ancillary->tag, WV_GRIBTAG)) {
	   for (j=0;j<ancillary->dims[2]*ancillary->dims[1];j++) ancillary->data[j] /= 10.0F;
        }
      

     return(MODIS_S_SUCCESS);
  }  /* else if (!strcmp(ancillary->anci_source, "GRIB\0")) */
  else {  /* ancillary->anci_source has garbarge or nothing; in all likelihood, 
             none of the ancillary data files for this type of data are available */
        return (MODIS_E_HDF_FAILURE);
       }
  
}
			

int read_EPT_array(FILE *oz_fp, int *nlat, int *nlon, float **foz_data) 
/*
!C**********************************************************************
!Description:   
  Reads data from EP-TOMS ASCII files.
 
!Input Parameters:
  FILE *oz_fp    -- File pointer (return value of fopen()).

!Output Parameters:
  int  *nlat     -- number of latitude points for TOMS data
  int  *nlon     -- number of longitude points for TOMS data
  float **foz_data

Global Variables:

!Revision History:
  1997/10/?? Neal Devine -- Initial Version
  
  Adapted from MOD_PROZN code, routine Read_TOMS_Ozone(), on 29-APR-04 by Jim Ray (SSAI).

!Team-unique Header:
  !References and Credits:
     Neal Devine (neal.devine@gsfc.nasa.gov)

     This software is developed by the MODIS Science Data Support
     Team for the National Aeronautics and Space Administration,
     Goddard Space Flight Center, under contract NAS5-32373.

     HDF portions developed at the National Center for Supercomputing
     Applications at the University of Illinois at Urbana-Champaign.

!Design Notes: 
   
PDL:
!END********************************************************************
*/
{
#define MAX_LINE  333
#define NLAT_MAX  181
#define NLON_MAX  360
  char msg[256];
  char location[50]="ancillary.c, read_EPT_array\0";
  int ret_pgs;
  char line[333],one_value[4];
  int i,j,ilat,ilon,ilon_in_line,yday,year;
  float32 a,b,c,d;
  long li,lj;
  int16 *data;

  /* Read first three lines of TOMS Ozone file
   *  line 1) day & year,
   *  line 2) # longitude bins.
   *  line 3) # latitude bins.
   */
  for ( i = 0 ; i < 3 ; i++ ) {
    if ( fgets(line, MAX_LINE, oz_fp) == NULL ) {
      sprintf(msg, "failed to read line %d of EP-TOMS Ozone file", i);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return( 1 );
    }
    switch (i) {
    case 0 : sscanf( line, "%*s %d %*s %*d, %d", &yday, &year ); break;
    case 1 : sscanf( line, "%*s %d", nlon ); break;
    case 2 : sscanf( line, "%*s %*s %d" , nlat ); break;
      /*    case 0 : sscanf( line, "Day: %d %*s %*d, %d", yday, year ); break;*/
      /*    case 1 : sscanf( line, "Longitudes: %d", nlon ); break;*/
      /*    case 2 : sscanf( line, "Latitudes : %d" , nlat ); break;*/
    }
  }

  /* Read the data which is written from south to north and west_to_east (most rapid).
   * The data is stored as 4char ascii with upto 25 values per line.
   * The output data is organized north to south (reversed from input).
   * The transformation of the data is from SeaWiFS code o3nrt.c (if i > 0...).
   */
  if ((data = (int16 *) malloc(sizeof(int16) * (*nlat) * (*nlon) )) == NULL) return(-1);
  for ( ilat = 0 ; ilat < *nlat ; ilat++ ) {
    for ( ilon = 0 ; ilon < *nlon ; ilon++ , ilon_in_line++) {
      if ( ilon % 25 == 0 ) {
	ilon_in_line = 0;
	if ( fgets(line, MAX_LINE, oz_fp) == NULL ) {
	  printf("ERROR: failed to read line with lat,ilon = %f,%d of TOMS Ozone file\n", 
		     (180./ *nlat)*(ilat+.5)-90., ilon);
	  return( 2 );
	}
      }
      strncpy( one_value, line+1+3*ilon_in_line, 3 );  /* necessary to handle "  0###" correctly. */
      if ( sscanf( one_value, "%3d", &i ) != 1 ) {
	printf("ERROR: failed to sscanf line with lat,ilon = %f,%d of TOMS Ozone file\n", 
		     (180./ *nlat)*(ilat+.5)-90., ilon);
	return( 4 );
      }
      if ( i > 0 ) data[ (*nlat-1-ilat)*(*nlon)+ilon ] = i;
      else         data[ (*nlat-1-ilat)*(*nlon)+ilon ] = 0;
    }
  }

  /* this converts it all to floats, and expands the column-wise dimension from
     288 (EP-TOMS default) to 360 (which matches GDAS, GRB, etc.) */
  if ((*foz_data = (float *) malloc(sizeof(float)*NLON_MAX*NLAT_MAX)) == NULL) return(-1);
            
  for(i=0;i<*nlat;i++) {
        li = i*(*nlon);
	lj = i*NLON_MAX;
        for(j=0;j<((*nlon)/4);j++) {
	    a = (float32)data[li + (j*4) + 0];
	    b = (float32)data[li + (j*4) + 1];
	    c = (float32)data[li + (j*4) + 2];
	    d = (float32)data[li + (j*4) + 3];
	    (*foz_data)[lj++] = a;
	    
            /* do NOT include zeros in this process */
	    if ((a != 0.0F) && (b != 0.0F)) (*foz_data)[lj++] = a*0.25F + b*0.75F;
	    else                            (*foz_data)[lj++] = 0.0F;
	    if ((c != 0.0F) && (b != 0.0F)) (*foz_data)[lj++] = b*0.50F + c*0.50F;
	    else                            (*foz_data)[lj++] = 0.0F;
	    if ((c != 0.0F) && (d != 0.0F)) (*foz_data)[lj++] = d*0.25F + c*0.75F;
	    else                            (*foz_data)[lj++] = 0.0F;

	    (*foz_data)[lj++] = d;
        }
     }
  
  free(data);
  *nlon = NLON_MAX;
  *nlat = NLAT_MAX;
  return( 0 );

}

		
float32 local_interpolate(int32 iline, int32 lines, int32 isamp, int32 samps, 
                          float32 rowfactor, float32 colfactor, float32 *data)
{
  float32 v1, v2, v3, v4;
  int32 i1, i2, i3, i4;
  float32 interp;
  /* obtain four corners */

  v1= 0.0;
  v2= 0.0;
  v3= 0.0;
  v4= 0.0;
  i1= iline * samps + isamp;
  i2= (iline + 1) * samps + isamp;
  i3= (iline + 1) * samps + isamp+1;
  i4= iline * samps + isamp+1;

  if (isamp+1 >= samps) {  /* 29-OCT-01 */
     i3= (iline + 1) * samps;
     i4= iline * samps; 
  }
  if (i1 >= 0) v1= data[i1];
  if (i2 >= 0) v2= data[i2];
  if (i3 >= 0) v3= data[i3];
  if (i4 >= 0) v4= data[i4];
  
  interp=(1.0F-rowfactor)*(1.0F-colfactor)*v1 +  
    rowfactor*(1.0F-colfactor)*v2 +
    rowfactor*colfactor*v3 + 
    colfactor*(1.0F-rowfactor)*v4;

  return(interp);

}

void clean_up_ancillary_data(void)
{
int i,j;
for(i=0;i<6;i++)
for(j=0;j<2;j++) 
   if ( tmpfltarray[i][j] != NULL) free(tmpfltarray[i][j]);
   
if(daowv.data!=NULL)free(daowv.data);
if(daoozone.data!=NULL)free(daoozone.data);
if(gdasozone.data!=NULL)free(gdasozone.data);
if(daospres.data!=NULL)free(daospres.data);
if(daotmp.data!=NULL)free(daotmp.data);
if(daouwnd.data!=NULL)free(daouwnd.data);
if(daovwnd.data!=NULL)free(daovwnd.data);
if(avg_z.data!=NULL)free(avg_z.data);
}


int32 get_cart_linesamp(float32 lat, float32 lon, float32 latres, int32 samps, 
		       float32 lonres, int32 *iline,
		       int32 *isamp, float32 *rowfactor, float32 *colfactor)
/*
!C*****************************************************************************
!Description:  Given a latitute and longitude, this function determines the 
               line, sample, subline factor and subsample factor of the point 
	       in the DAO geographic grid.

!Input Parameters:
lat: latitude of point in question
lon: longitude of point in question
latres: latitude resolution of the input data
lonres: longitude resolution of the input data
samps: columns in the input data file

!Output Parameters:
iline: line number containing the point in question
isamp: sample number containing the point in question
rowfactor: vertical distance of the point in question from the 
           center of iline to the center of the next southern line.
colfactor: horizontal distance of the point in question from the 
           center of isamp to the center of the next eastern pixel.

!Input/Output Parameters:

!Returns:
	0	failure (lat and/or lon is out of range)
	1	success

!Revision History:
        Revision 1.1, August 1997, P. Fisher
	1.  Added absolute value call into rowfactor/colfactor calculation
            (bug fix).

	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes
!END***************************************************************************
*/
{

  float32 rowsub, colsub;
  float64 tmp;

  /* check inputs */
  if(lat>90.0F||lat<-90.0F||lon>180.0F||lon<-180.0F)return(0);

  *iline = (int32)(((90.0F - lat)/latres)+0.5F);
  *rowfactor = (float32)((90.0F - ((float32)(*iline)*latres)) - lat)/latres;
  if((*rowfactor)<0.0F){
    (*iline)--;
    *rowfactor=1.0F-fabsf((float)(*rowfactor));
  }

  /* special wrap around case */
  if(lon>(180.0F-(lonres*0.5F)))lon=-180.0F-(180.0F-lon);

  *isamp = (int32)(((lon +180.0F)/lonres) + 0.5F);
  tmp = (float64)(lon +(180.0F - ((*isamp)*lonres)))/lonres; /*29-OCT-01*/
  *colfactor = (float32)tmp;
  if((*colfactor)<0.0F){
    (*isamp)--;
    /* special wrap around case */
    if((*isamp)<0)*isamp=(samps-1);

    *colfactor=1.0F-fabsf((float)(*colfactor));
    }
  
  return(1);
}


float32 get_cart_interp(int32 iline, int32 isamp, int32 samps, float32 rowfactor, 
		     float32 colfactor, d_ancillary *anc)
/*
!C*****************************************************************************
!Description:  bilinearly interpolates a point in question through its 
               isamp, iline, row and collumn subsample distances 
	       (supplied through a call to get_cart_linesamp()) for data 
	       in the DAO geographic grid.

!Input Parameters:
iline: line number containing the point in question
isamp: saple number containing the point in question
smaps: number of collumns in the input DAO data file
rowfactor: vertical distance of the point in question from the 
           center of iline to the center of the next southern line.
colfactor: horizontal distance of the point in question from the 
           center of isamp to the center of the next eastern pixel.
anc: ancillary data structure

!Output Parameters: none

!Input/Output Parameters: none

!Returns: interpolated data value

!Revision History:
	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).


!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes
!END****************************************************************************/
{
  float32 v1, v2, v3, v4;
  int32 i1, i2, i3, i4;
  float32 interp;
  
  /* obtain four corners */
  v1= 0.0;
  v2= 0.0;
  v3= 0.0;
  v4= 0.0;
  i1= iline * samps + isamp;
  i2= (iline + 1) * samps + isamp;
  i3= (iline + 1) * samps + isamp+1;
  i4= iline * samps + isamp+1;
  if (i1 >= 0) v1= anc->data[i1];
  if (i2 >= 0) v2= anc->data[i2];
  if (i3 >= 0) v3= anc->data[i3];
  if (i4 >= 0) v4= anc->data[i4];

/*  v1 = anc->data[iline * samps + isamp];
  v2 = anc->data[(iline + 1) * samps + isamp];
  v3 = anc->data[(iline + 1) * samps + isamp+1];
  v4 = anc->data[iline * samps + isamp+1];
 */
  interp=(1.0F-rowfactor)*(1.0F-colfactor)*v1 +  
    rowfactor*(1.0F-colfactor)*v2 +
    rowfactor*colfactor*v3 + 
    colfactor*(1.0F-rowfactor)*v4;

  return(interp);

}

int map_ancillary_data(float *lat, float *lon, int total_1km_pixels, float *O3, float *wv, float *sp, float *t2m, float *uw, float *vw, 
                       int *bands, uint16 *qa250, uint32 *qa500, uint32 *qa1)
/*
!C****************************************************************************

!Routine: map_ancillary_data()

!Description: Maps various ancillary data parameters into scan space.

!Input Parameters:
  float *lat
  float *lon
  int total_1km_pixels
    
!Input/Output Parameters:
  float *O3         }
  float *wv         } the parameters being mapped
  float *sp         } in:empty (or with data to be ignored)
  float *t2m        } out:full
  float *uw         } 
  float *vw         } 
  int16 *qa250       }
  int32 *qa500       } filled with USEDATMCONSTANT if we have a bad interpolation.
  int32 *qa1         }
  
!Output Parameters:
  none

!Returns:
  0
  
!Revision History:
  Revision 1.0  01-APR-2003
  Jim Ray

!Team-unique Header:
  This software was developed by:

    MODIS Science Data Support Team for the National Aeronautics and Space 
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

  Developer:
      Jim Ray
      Science Systems and Applications, Inc.
      MODIS Land Science Team
      NASA Goddard Space Flight Center
      Greenbelt, Maryland 20771
      jim@cratmos.gsfc.nasa.gov

!References and Credits 

!Design Notes: 
            called by routine atmcorrect() (atmcor.c)
	    Needs a whole lot of work.
	    Maps data in the global variables daoozone, daowv, daospres, daotmp.

!END**************************************************************************/
{ 
int index1km, index250m, index500m, ich, row, col, xrow, xcol;
int NSP = (int)(NBSPOTS_1KM);  
int NSC = (int)(NBSCANS_1KM);  
int32 iline, isamp;
char location[100]="ancillary.c, map_ancillary_data\0";
float32 rowfactor, colfactor;
float32 get_cart_interp(int32 iline, int32 isamp, int32 samps, float32 rowfactor, float32 colfactor, d_ancillary *anc);
int32 get_cart_linesamp(float32 lat, float32 lon, float32 latres, int32 samps, float32 lonres, int32 *iline,
		       int32 *isamp, float32 *rowfactor, float32 *colfactor);

index1km = 0;		       
for (row=0;row<NSC;row++) 
for (col=0;col<NSP;col++) {

      if(!(get_cart_linesamp(lat[index1km], lon[index1km], daoozone.latres, daoozone.dims[2],
			     daoozone.lonres, &(iline), &(isamp), &(rowfactor), &(colfactor)))){
	modsmf(MODIS_E_INTERP_ERROR, "ancillary data, continuing using default constants", location);
        O3[index1km] = OZONEDEFAULT;
        wv[index1km] = WATERVAPORDEFAULT;
        sp[index1km] = SURFACEPRESSUREDEFAULT;
        t2m[index1km] = 280.0F;
        uw[index1km] = 7.0F;
        vw[index1km] = 7.0F;

        for(ich=BAND1_250M;ich<BAND1_500M;ich++){
	   if((bands[ich])&&(qa250 != NULL)){
	      for (xrow = (row*4); xrow < (row*4)+4; xrow++) {
	          index250m = xrow*NSP*4;
	          for (xcol = (col*4); xcol < (col*4)+4; xcol++) 
	              qa250[index250m+xcol]|=BANDQA250(USEDATMCONSTANT, ich);
	          }
	      }
         }
      
        for(ich=BAND1_500M;ich<BAND1_1KM;ich++){
	   if((bands[ich])&&(qa500 != NULL)){
	      for (xrow = (row*2); xrow < (row*2)+2; xrow++) {
	          index500m = xrow*NSP*2;
	          for (xcol = (col*2); xcol < (col*2)+2; xcol++) 
	              qa500[index500m+xcol]|=BANDQA500(USEDATMCONSTANT, ich);
	          }
	      }
         }
            
        for(ich=BAND1_1KM;ich<BAND8;ich++){
	   if((bands[ich])&&(qa1 != NULL))qa1[index1km]|=BANDQA1(USEDATMCONSTANT, ich);
         }

	       }
      else {
        O3[index1km] = get_cart_interp(iline, isamp, daoozone.dims[2], rowfactor, colfactor, &daoozone) * 0.001;
        wv[index1km] = get_cart_interp(iline, isamp, daoozone.dims[2], rowfactor, colfactor, &daowv);
        sp[index1km] = get_cart_interp(iline, isamp, daoozone.dims[2], rowfactor, colfactor, &daospres);
        t2m[index1km] = get_cart_interp(iline, isamp, daoozone.dims[2], rowfactor, colfactor, &daotmp);
        uw[index1km] = get_cart_interp(iline, isamp, daoozone.dims[2], rowfactor, colfactor, &daouwnd);
        vw[index1km] = get_cart_interp(iline, isamp, daoozone.dims[2], rowfactor, colfactor, &daovwnd);
         }

     index1km++;
      }
      
return(0);
}



