/* To escape from the lhdf library overhead, our own hdf libraries */
#include <stdio.h>
#include <stdlib.h>

#include "modis_srefl.h"
#include "protos.h"


int HDF_Read_2D(int32 sds_id, int iline, int nline, int samps, void *data)
/*
!C*****************************************************************************

!Description: 
    HDF_Open:  Reads a 2D SDS.  
    Part of the MOD09 HDF utility library.


!Input Parameters:
sds_id, iline, nline, samps

!Output Parameters:
data

!Input/Ouput Parameters:
none

!Returns:
0 on failure, 1 on success


!Revision History:

Revision 1.0, P. Fisher, July 1998
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-96062

!References and Credits:
  
    MODIS Science Team Members:
			
	Eric F Vermote
	Univerisity of MD.
   	NASA's GSFC Code 923
   	Greenbelt, MD , 20771
	eric@kratmos.gsfc.nasa.gov
	301-286-6232

    Developers:
      Paul Fisher
      MODIS Land Science Team           Science Systems and Applications Inc.
      fisher@ltpmail.gsfc.nasa.gov      NASA's GSFC Code 923 
      phone (301) 286-2980              Greenbelt, MD 20771
 

!Design Notes

!END
****************************************************************************/

{

  char location[50]="hdflib.c, HDF_Read_2D\0";
  int32 start[2], edge[2];

  start[0]=iline;
  start[1]=0;

  edge[0]=nline;
  edge[1]=samps;

  if(SDreaddata(sds_id, start, NULL, edge, data)==FAIL){
    modsmf(MODIS_E_HDF_READ, "Error reading SDS data.", location);
    return(0);
  }
  return(1);
}


int HDF_Write_2D(int32 sds_id, int iline, int nline, int samps, void *data)
/*
!C*****************************************************************************

!Description: 
    HDF_Open:  Reads a 2D SDS.  
    Part of the MOD09 HDF utility library.


!Input Parameters:
sds_id, iline, nline, samps

!Output Parameters:
data

!Input/Ouput Parameters:
none

!Returns:
0 on failure, 1 on success


!Revision History:

Revision 1.0, P. Fisher, July 1998
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-96062

!References and Credits:
  
    MODIS Science Team Members:
			
	Eric F Vermote
	Univerisity of MD.
   	NASA's GSFC Code 923
   	Greenbelt, MD , 20771
	eric@kratmos.gsfc.nasa.gov
	301-286-6232

    Developers:
      Paul Fisher
      MODIS Land Science Team           Science Systems and Applications Inc.
      fisher@ltpmail.gsfc.nasa.gov      NASA's GSFC Code 923 
      phone (301) 286-2980              Greenbelt, MD 20771
 

!Design Notes

!END
****************************************************************************/

{

  char location[50]="hdflib.c, HDF_Write_2D\0";
  int32 start[2], edge[2];

  start[0]=iline;
  start[1]=0;

  edge[0]=nline;
  edge[1]=samps;

  if(SDwritedata(sds_id, start, NULL, edge, data)==FAIL){
    modsmf(MODIS_E_HDF_READ, "Error writing SDS data.", location);
    return(0);
  }
  return(1);
}


void writeHDFSDSattributes_byte(int32 sds_id, int8 fillv, int8 vrangeh,
			   int8 vrangel, float64 scale, float64 offset, char *errstring, 
			   int32 cal, char *units, char *long_name)
/*
!C*****************************************************************************
!Description: writes native HDF attributes (_FillValue, valid_range, and 
              calibration attributes) to an SDS.

!Input Parameters:
hd: Lhdf_file_t file handle
param: Lhdf_param_t hdf parameter type
fillv: fill value
vrangeh: high valid range value
vrangel: low valid range value
scale: scale_factor value
offset: add_offset value
errstring: string used to pass the name of the SDS being written to to the 
           SMF functions (free form, may be used to pass any information).
cal: switch that determines if the calibration attributes should be written 
     (if TRUE write calibration parameters)

!Output Parameters: none

!Input/Output Parameters: none

!Returns: none

!Revision History:
	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes
!END***************************************************************************
*/
{

  char location[100]="hdflib.c, writeHDFSDSattributes_byte\0";
  char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */

  if(SDsetrange(sds_id, &vrangeh, &vrangel)==FAIL){
    sprintf(msg, "Unable to write valid range HDF attributes for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    

 
  if(SDsetfillvalue(sds_id, &fillv)==FAIL){
    sprintf(msg, "Unable to write fill value HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    

  if(cal){
    if(SDsetcal(sds_id, scale, 0.0, offset, 0.0, DFNT_FLOAT32)==FAIL){
      sprintf(msg, "Unable to write calibration HDF attributes for the %s", errstring);
      modsmf(MODIS_E_HDF_ERROR, msg, location);
      return;
    }  
  }

  if(SDsetattr(sds_id, "units\0", DFNT_CHAR, (int32)(strlen(units)), units)==FAIL){
    sprintf(msg, "Unable to write units HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    
   if(SDsetattr(sds_id, "long_name\0", DFNT_CHAR, (int32)(strlen(long_name)), long_name)==FAIL){
    sprintf(msg, "Unable to write long_name HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    

  return;
}

void writeHDFSDSattributes(int32 sds_id, int16 fillv, int16 vrangeh,
			   int16 vrangel, float64 scale, float64 offset, char *errstring, 
			   int32 cal, char *units, char *long_name)
/*
!C*****************************************************************************
!Description: writes native HDF attributes (_FillValue, valid_range, and 
              calibration attributes) to an SDS.

!Input Parameters:
hd: Lhdf_file_t file handle
param: Lhdf_param_t hdf parameter type
fillv: fill value
vrangeh: high valid range value
vrangel: low valid range value
scale: scale_factor value
offset: add_offset value
errstring: string used to pass the name of the SDS being written to to the 
           SMF functions (free form, may be used to pass any information).
cal: switch that determines if the calibration attributes should be written 
     (if TRUE write calibration parameters)

!Output Parameters: none

!Input/Output Parameters: none

!Returns: none

!Revision History:
	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes
!END***************************************************************************
*/
{

  char location[100]="hdflib.c, writeHDFSDSattributes\0";
  char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */

  if(SDsetrange(sds_id, &vrangeh, &vrangel)==FAIL){
    sprintf(msg, "Unable to write valid range HDF attributes for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    

 
  if(SDsetfillvalue(sds_id, &fillv)==FAIL){
    sprintf(msg, "Unable to write fill value HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    

  if(cal){
    if(SDsetcal(sds_id, scale, 0.0, offset, 0.0, DFNT_FLOAT32)==FAIL){
      sprintf(msg, "Unable to write calibration HDF attributes for the %s", errstring);
      modsmf(MODIS_E_HDF_ERROR, msg, location);
      return;
    }  
  }

  if(SDsetattr(sds_id, "units\0", DFNT_CHAR, (int32)(strlen(units)), units)==FAIL){
    sprintf(msg, "Unable to write units HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    
   if(SDsetattr(sds_id, "long_name\0", DFNT_CHAR, (int32)(strlen(long_name)), long_name)==FAIL){
    sprintf(msg, "Unable to write long_name HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    

  return;
}

void writeHDFSDSattributes_float(int32 sds_id, float32 fillv, float32 vrangeh,
			float32 vrangel, float32 scale, float32 offset, char *errstring, 
			int32 cal, char *units, char *long_name)
/*
!C*****************************************************************************
!Description: writes native HDF attributes (_FillValue, valid_range, and 
              calibration attributes) to an SDS.  Modified from writeSDSattributes
	      to handle floating point inputs.

!Input Parameters:
hd: Lhdf_file_t file handle
param: Lhdf_param_t hdf parameter type
fillv: fill value
vrangeh: high valid range value
vrangel: low valid range value
scale: scale_factor value
offset: add_offset value
errstring: string used to pass the name of the SDS being written to to the 
           SMF functions (free form, may be used to pass any information).
cal: switch that determines if the calibration attributes should be written 
     (if TRUE write calibration parameters)

!Output Parameters: none

!Input/Output Parameters: none

!Returns: none

!Revision History:
	Original version 1998, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes
!END***************************************************************************
*/
{
 
  char location[100]="hdflib.c, writeHDFSDSattributes_float\0";
  char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */

  if(SDsetrange(sds_id, &vrangeh, &vrangel)==FAIL){
    sprintf(msg, "Unable to write valid range HDF attributes for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    
  
  
  if(SDsetfillvalue(sds_id, &fillv)==FAIL){
    sprintf(msg, "Unable to write fill value HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    
  
  if(cal){
    if(SDsetcal(sds_id, scale, 0.0, offset, 0.0, DFNT_FLOAT32)==FAIL){
      sprintf(msg, "Unable to write calibration HDF attributes for the %s", errstring);
      modsmf(MODIS_E_HDF_ERROR, msg, location);
      return;
    }  
  }
   if(SDsetattr(sds_id, "units\0", DFNT_CHAR, (int32)(strlen(units)), units)==FAIL){
    sprintf(msg, "Unable to write units HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    
   if(SDsetattr(sds_id, "long_name\0", DFNT_CHAR, (int32)(strlen(long_name)), long_name)==FAIL){
    sprintf(msg, "Unable to write long_name HDF attribute for the %s", errstring);
    modsmf(MODIS_E_HDF_ERROR, msg, location);
    return;
  }    
 
  return;
}


