#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mfhdf.h"
#define FLT_MAX 9999999.0
#include "PGS_MODIS_37092.h"
#include "modis_srefl.h"
#include "struct.h"
#include "param.h"

int get_params(commands_t *commands, short *TESTLINE, short *TESTPIX, short int fullstemp,
		int32 *scans_per_gran, PGSt_double *orbit_start_tai, 
		PGSt_double *orbit_end_tai, int32 *process_bands, char *report)
/*
!C*****************************************************************************
!Description:  get_params()
	Reads run-time parameters from the PCF.

!Input Parameters:
	fullstemp              Currently hard-coded to 1 (not used)
	
!Output Parameters:
	commands               structure holding most of the parameters.
	fatool                 PGSt_IO_Gen_FileHandle for ATOOLS filename (LUN 209711)      
	TESTLINE               ETOOL stuff, not currently used  (LUN 209604)
	TESTPIX                same                             (LUN 209605)
        scans_per_gran         the number of scans to process (LUN 209930)
	orbit_start_tai        Starttime of orbit in TAI-format (LUN 10258)
	orbit_end_tai          Endtime of orbit in TAI-format (LUN 10259)
	process_bands          Flags for processing each band (LUNs 209900 - 209915).
	
!Input/Output Parameters:
        report                 A description of what the LUN being read is for, 
	                       and the value read (for output to the LogReport file).

!Returns:
        2 - error
        0 - success

!Revision History:

Revision 1.0, J. Ray, June, 2001
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-96062

!References and Credits:
  
    MODIS Science Team Members:
			
	Eric F Vermote
	Univerisity of MD.
   	NASA's GSFC Code 923
   	Greenbelt, MD , 20771
	eric@kratmos.gsfc.nasa.gov
	301-614-5521

    Developers:
      Jim Ray
      MODIS Land Science Team           Science Systems and Applications Inc.
      jim@cratmos.gsfc.nasa.gov         NASA's GSFC Code 923 
      phone (301) 614-6684              Greenbelt, MD 20771
 

!Design Notes

        Pulled out of main() to make main() more readable.
        Variable sat_type is set -- it is global.
	
!END***************************************************************************
*/
{
#define PLAT_LENGTH 8
int i;
char sat_instr[PGS_SMF_MAX_MSG_SIZE] = {'\0'};
char msg[PGS_SMF_MAX_MSG_SIZE] = {'\0'};
char location[50]="params.c, get_params\0";
char configParamVal[PGSd_PC_VALUE_LENGTH_MAX] = {'\0'};
PGSt_SMF_status ret;
/*PGSt_IO_Gen_FileHandle *OpenFileWrite(PGSt_PC_Logical logical, PGSt_integer version);*/
void lun_description_lut(char *report, PGSt_PC_Logical number, char *param_value);
char loc_platform[PLAT_LENGTH];

/* these are declared global to main() in modis_srefl.c */
extern int AT_SCF;
extern float32 ALPHAF, BETAF, GAMMAF, DELTAF, MIRAFTOG, MIRAFTIG, TATOG, TATIG, ROGT;
extern int SCFSTARTSCAN, SCFENDSCAN, sat_type;
#ifdef SUBSET
extern float32 T2M_value, NCEPH2O_value;
#endif
extern char *ProcessingEnvironment, *Platform;

AT_SCF = 0;

  /* get user config params */
  sat_type = 0;  /* terra */
  if(PGS_PC_GetConfigData(SATELLITEINSTRUMENTLUN,sat_instr)==PGS_S_SUCCESS)
  { 
    lun_description_lut(report, SATELLITEINSTRUMENTLUN, sat_instr);
    
    /* New, 03-AUG-00: for Terra or Aqua switching. */
    for (i=0;i<5;i++)sat_instr[i] = toupper(sat_instr[i]);
    if (strstr(sat_instr,"AM1M"))      sat_type = 0;  /* terra */     
    else if (strstr(sat_instr,"PM1M")) sat_type = 1;  /* aqua */
    else if (strstr(sat_instr,"AP1M")) {
       modsmf(MODIS_W_FUNCTIONALITY_NOT_AVAIL,  
          "PGE11 cannot perform mixed Terra/Aqua processing", location);
       return(2);
    }
    else {
       sprintf(msg, "'SATELLITEINSTRUMENTLUN', %d,", SATELLITEINSTRUMENTLUN);
       modsmf(MODIS_E_BAD_PARAMETER_READ, msg, location);
    }
  }  /* else: fail quietly */
  
  if(PGS_PC_GetConfigData(MOD09_ADJACENCY,configParamVal)==PGS_S_SUCCESS) {
    commands->adja_on=atoi(configParamVal);
    lun_description_lut(report, MOD09_ADJACENCY, configParamVal);
    }
  
  if(PGS_PC_GetConfigData(MOD09_BRDF,configParamVal)==PGS_S_SUCCESS){
    commands->brdf_mode = atoi(configParamVal);
    lun_description_lut(report, MOD09_BRDF, configParamVal);
    if (commands->brdf_mode != BRDF_BOSTON && commands->brdf_mode != BRDF_MONTANA) 
      commands->brdf_mode = BRDF_NONE;
  }

  if(PGS_PC_GetConfigData(MOD09_LAM,configParamVal)==PGS_S_SUCCESS){
    commands->ref_lam = atoi(configParamVal);
    if (commands->ref_lam != 0)commands->ref_lam=1;
    lun_description_lut(report, MOD09_LAM, configParamVal);
  }

  /* new, 27-FEB-02 */
  commands->makegcm = 0;
  if(PGS_PC_GetConfigData(MAKECMGLUN,configParamVal)==PGS_S_SUCCESS){
    commands->makegcm = atoi(configParamVal);
    if (commands->makegcm != 0)commands->makegcm=1;
#ifdef MOD02CRS_PROC
    /* if we are in MOD02CRS mode, the reflectance output file LUN is 209010, 
       which is the same as one of the CMG files!  Also, CMG processing is not
       going to work with 5km inputs anyway, so disable CMG output for MOD02CRS 
       processing (30-APR-02) */
    commands->makegcm=0;
#endif
  }

  /* New, 19-FEB-99 *
     16-MAR-99: No need to read whether/not to do atmcor over clouds; will 
     do atmcor over clouds all the time.  (Jim Ray, SSAI.)
  if(PGS_PC_GetConfigData(MOD09_LAMC,configParamVal)==PGS_S_SUCCESS)
    lun_description_lut(report, MOD09_LAMC, configParamVal);
    commands->ref_lamc = atoi(configParamVal);
    if (commands->ref_lamc != 0)commands->ref_lamc=1;
  
  * End, new stuff */
  commands->ref_lamc=1;


#ifdef ETOOL  
  if(PGS_PC_GetConfigData(MOD09_DEBUGLINE,configParamVal)==PGS_S_SUCCESS){
    *TESTLINE = atoi(configParamVal);
  } 
  if(PGS_PC_GetConfigData(MOD09_DEBUGPIX,configParamVal)==PGS_S_SUCCESS){
    *TESTPIX = atoi(configParamVal);
  }
  
  /* if one is bad, disable * /
  if (((*TESTPIX == -1)&&(*TESTLINE != -1))||((*TESTPIX != -1)&&(*TESTLINE == -1))) {
      printf("Warning: one or both of the requested line/pixel for debugging -- %d, %d -- is bad.\n",
           *TESTLINE,* TESTPIX);
      printf(" Will disable.\n");
      *TESTLINE = *TESTPIX = -1;
  }
  
  if (*TESTPIX > NBSPOTS_1KM) {
      printf("Warning: requested pixel for debugging -- %d -- is beyond %d, the number of\n", 
           *TESTPIX, NBSPOTS_1KM);
      printf(" pixels in a 1 km scan line.  Will set requested pixel to %d.\n", NBSPOTS_1KM);
      *TESTPIX = NBSPOTS_1KM;
  }
  
  if (*TESTLINE != -1)
      printf("Will echo 1 km values at row/pix %d, %d\n", *TESTLINE, *TESTPIX); */
#endif

  if(PGS_PC_GetConfigData(MOD09_COARSE,configParamVal)==PGS_S_SUCCESS){ 
    commands->coarse_product = atoi(configParamVal);
    if(commands->coarse_product != 0)commands->coarse_product=1;
    lun_description_lut(report, MOD09_COARSE, configParamVal);
  }

#ifdef MOD02CRS_PROC
    if(commands->coarse_product == 1) {
       modsmf(MODIS_W_INFORMATION , "cannot derive coarse product from 5 km input,"
              " will disable user-requested coarse product", location);
       commands->coarse_product=0;
         }
#endif

  commands->make_cloud_shadow = 1;   /* default!!! */
  if(PGS_PC_GetConfigData(MAKECLDSHD,configParamVal)==PGS_S_SUCCESS){
    commands->make_cloud_shadow = atoi(configParamVal);
    if(commands->make_cloud_shadow != 1)commands->make_cloud_shadow=0;
    lun_description_lut(report, MAKECLDSHD, configParamVal);
  }

  if(PGS_PC_GetConfigData(USEMODAOT,configParamVal)==PGS_S_SUCCESS){
    commands->use_modaot = atoi(configParamVal);
    if(commands->use_modaot != 0)commands->use_modaot=1;
    lun_description_lut(report, USEMODAOT, configParamVal);
  }

#ifdef ATOOL
  if(PGS_PC_GetConfigData(ATOOLLUN,configParamVal)==PGS_S_SUCCESS){
    commands->atool=atoi(configParamVal);
    /* default is off */
    if(commands->atool != 1)commands->atool=0;
  }
#else
  commands->atool=0;
#endif

  if(commands->atool){

    if(PGS_PC_GetConfigData(ATOOLSPOT,configParamVal)==PGS_S_SUCCESS){
      commands->atoolspot=atoi(configParamVal);
    }
    if(commands->atoolspot<1 || commands->atoolspot> NBSPOTS_1KM)commands->atoolspot=1;
    if(PGS_PC_GetConfigData(ATOOLSCAN,configParamVal)==PGS_S_SUCCESS){
      commands->atoolscan=atoi(configParamVal);
    }
    if(commands->atoolscan<1 || commands->atoolscan> NBSCANS_1KM)commands->atoolscan=1;
  }


  if(PGS_PC_GetConfigData(SCANS_PER_GRAN,configParamVal)==PGS_S_SUCCESS) {
    *scans_per_gran = atoi(configParamVal);  
    lun_description_lut(report, SCANS_PER_GRAN, configParamVal);
  }

  if(PGS_PC_GetConfigData(PROCESS_SEA,configParamVal)==PGS_S_SUCCESS){
    commands->process_sea = atoi(configParamVal);
    if(commands->process_sea != 1)commands->process_sea=0;
    lun_description_lut(report, PROCESS_SEA, configParamVal);
  }

  /* 24-SEP-01 */
  if(PGS_PC_GetConfigData(ALPHAFLUN,configParamVal)==PGS_S_SUCCESS) {
    ALPHAF = (float32)atof(configParamVal);
    lun_description_lut(report, ALPHAFLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(BETAFLUN,configParamVal)==PGS_S_SUCCESS) {
    BETAF = (float32)atof(configParamVal);
    lun_description_lut(report, BETAFLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(GAMMAFLUN,configParamVal)==PGS_S_SUCCESS) {
    GAMMAF = (float32)atof(configParamVal);
    lun_description_lut(report, GAMMAFLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(DELTAFLUN,configParamVal)==PGS_S_SUCCESS) {
    DELTAF = (float32)atof(configParamVal);
    lun_description_lut(report, DELTAFLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(MIRAFTOGLUN,configParamVal)==PGS_S_SUCCESS) {
    MIRAFTOG = (float32)atof(configParamVal);
    lun_description_lut(report, MIRAFTOGLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(MIRAFTIGLUN,configParamVal)==PGS_S_SUCCESS) {
    MIRAFTIG = (float32)atof(configParamVal);
    lun_description_lut(report, MIRAFTIGLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(TATOGLUN,configParamVal)==PGS_S_SUCCESS) {
    TATOG = (float32)atof(configParamVal);
    lun_description_lut(report, TATOGLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(TATIGLUN,configParamVal)==PGS_S_SUCCESS) {
    TATIG = (float32)atof(configParamVal);
    lun_description_lut(report, TATIGLUN, configParamVal);
    }
  if(PGS_PC_GetConfigData(ROGTLUN,configParamVal)==PGS_S_SUCCESS) {
    ROGT = (float32)atof(configParamVal);
    lun_description_lut(report, ROGTLUN, configParamVal);
    }

  /* 13-FEB-02 */
  strcpy(Platform,PLATFORM);  /* PLATFORM is defined at compile-time */
                              /* Platform is designed to reflect actual contents of
			         ProcessingEnvironment string from PCF */
  if(PGS_PC_GetConfigData(PLATFORMLUN,ProcessingEnvironment)!=PGS_S_SUCCESS){
      sprintf(msg, "LUN: %d", PLATFORMLUN);
      modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
      strcpy(ProcessingEnvironment,PLATFORM);  /* backup */
      strcpy(loc_platform,PLATFORM);  
    }
  else {
     for(i=0;i<PLAT_LENGTH;i++) {
        if (ProcessingEnvironment[i] == ' ') break;
        loc_platform[i] = toupper(ProcessingEnvironment[i]);
     }
     loc_platform[i] = '\0';
     /* Check to make sure PLATFORM (or something similar) is contained in 
        ProcessingEnvironment; if not, update Platform and inform user (complain). */
     if (!strcmp(PLATFORM, "HPUX")) { 
         if (  (strstr(loc_platform, "HP-UX") == NULL) ) {
              sprintf(msg, ", mismatch: '%s' vs. '%s'", PLATFORM, ProcessingEnvironment);
              modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
              strcpy(Platform,loc_platform); 
	     
	 }
     }
     if (!strcmp(PLATFORM, "IRIS4")) { 
         if (  (strstr(loc_platform, "IRI") == NULL) ) {
              sprintf(msg, ", mismatch: '%s' vs. '%s'", PLATFORM, ProcessingEnvironment);
              modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
              strcpy(Platform,loc_platform); 
	     
	 }
     }
     if (!strcmp(PLATFORM, "LINUX")) { 
         if (  (strstr(loc_platform, "LINUX") == NULL) ) {
              sprintf(msg, ", mismatch: '%s' vs. '%s'", PLATFORM, ProcessingEnvironment);
              modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
              strcpy(Platform,loc_platform); 
	     
	 }
     }

  }
  
  /* 29-APR-04: better SCF recognition */
  if ((strstr(ProcessingEnvironment, "patmos")) || (strstr(ProcessingEnvironment, "tracer")))
      AT_SCF = 1;
  else 
      AT_SCF = 0;
  
  if (AT_SCF == 1) {  /* at the land SCF */
    if(PGS_PC_GetConfigData(SCFSTARTSCANLUN,configParamVal)==PGS_S_SUCCESS) {
      SCFSTARTSCAN = (int)atoi(configParamVal);
      }
    if(PGS_PC_GetConfigData(SCFENDSCANLUN,configParamVal)==PGS_S_SUCCESS) {
      SCFENDSCAN = (int)atoi(configParamVal);
      }
  }
  
  /* 29-APR-04: ABSOLUTELY turn on the cloud shadow algorithm if you are NOT at the SCF.
     Make turning OFF the cloud shadow algorithm something that can ONLY be done at the SCF. */
  if (AT_SCF == 0) commands->make_cloud_shadow = 1;
  
#ifdef SUBSET
  /* FOR THIS VERSION OF THE CODE: turn off the coarse product( MOD09CRS is difficult/impossible 
     if the L1B is only 50 pixels on one side!). */
  commands->coarse_product = 0;
  /* AND, get T2M and NCEP H2O values from PCF parameters (for the cloud shadow)!!! */
  if(PGS_PC_GetConfigData(T2MLUN,configParamVal)==PGS_S_SUCCESS) {
    T2M_value = (float32)atof(configParamVal);
    }
  if(PGS_PC_GetConfigData(NCEPH2OLUN,configParamVal)==PGS_S_SUCCESS) {
    NCEPH2O_value = (float32)atof(configParamVal);
    }  
#endif
 
  /* initialize atool * /
  if(commands->atool){
    if(( *fatool = OpenFileWrite(ATOOLFNAME, 1))==NULL){
      /  * if there is a problem opening the file quit *  /
      modsmf(MODIS_F_OPEN_GEN_FILE, "atool file", location);
    }else{
      fprintf(*fatool, "#line_in_scan sample lon lat aot550 spres sz sa vz va refb1 refb2 refb3 refb4 refb5 refb6 refb7\n");
    }
  }

  / *  obtain orbit times */
  if(PGS_PC_GetConfigData(ORBITSTARTTIME, configParamVal)!=PGS_S_SUCCESS){
    modsmf(MODIS_W_BAD_PROCESSING_TIME, "the orbit start time", location);    
  }else{
    ret=PGS_TD_timeCheck(configParamVal);
    if(ret!=PGS_S_SUCCESS && ret!=PGSTD_M_ASCII_TIME_FMT_B){
      modsmf(MODIS_W_BAD_PROCESSING_TIME, "the orbit start time", location);
    }else{
      if(PGS_TD_UTCtoTAI(configParamVal, orbit_start_tai)!=PGS_S_SUCCESS){
	modsmf(MODIS_E_SDP_ERROR, "PGS_TD_UTCtoTAI()", location);
	modsmf(MODIS_W_BAD_PROCESSING_TIME, "the orbit start time", location);
      }else{
	/* expand acceptable range, as pcf times are not specifically precise
	   150 seconds is 1/2 of a granule */
	*orbit_start_tai-=150.0;
      }
    }
  }

  /* obtain orbit times */
  if(PGS_PC_GetConfigData(ORBITENDTIME, configParamVal)!=PGS_S_SUCCESS){
    modsmf(MODIS_W_BAD_PROCESSING_TIME, "the orbit end time", location);    
  }else{
    ret=PGS_TD_timeCheck(configParamVal);
    if(ret!=PGS_S_SUCCESS && ret!=PGSTD_M_ASCII_TIME_FMT_B){
      modsmf(MODIS_W_BAD_PROCESSING_TIME, "the orbit end time", location);
    }else{
      if(PGS_TD_UTCtoTAI(configParamVal, orbit_end_tai)!=PGS_S_SUCCESS){
	modsmf(MODIS_E_SDP_ERROR, "PGS_TD_UTCtoTAI()", location);
	modsmf(MODIS_W_BAD_PROCESSING_TIME, "the orbit end time", location);
      }else{
	/* expand acceptable range, as pcf times are not specifically precise
	   150 seconds is 1/2 of a granule */
	*orbit_end_tai+=150.0;
      }
    }
  }
  
  /* select which bands to process, and set up the process_bands[] array.  */  
  for(i=0;i<BAND8;i++) {
    if(PGS_PC_GetConfigData((PGSt_PC_Logical)(MOD09_BANDSEQSTART+i),
			    configParamVal)==PGS_S_SUCCESS) {
       process_bands[i]=atoi(configParamVal);
       lun_description_lut(report, (PGSt_PC_Logical)(MOD09_BANDSEQSTART+i), configParamVal);
       }
    }
   process_bands[BAND1_500M]=1;
   process_bands[BAND2_500M]=1;    /* these must ALWAYS be 1 */

sprintf(report+strlen(report),"\n   Processing Environment =   '%s'\n", ProcessingEnvironment);

return(0);
}

void lun_description_lut(char *report, PGSt_PC_Logical number, char *param_value)
{
#include "qa_map.h"
int i;
for (i=0;i<n_luns;i++) {   /* n_luns, LUNs and LUN description are in qa_map.h */
      if(luns[i] == number)
         sprintf(report+strlen(report),"  %d|%s|%s\n", (int)number,lun_description[i],param_value);
      }
}

void set_processing_flags(commands_t *commands, int32 *process_bands)
/*
!C*****************************************************************************
!Description:  set_processing_flags()
	Adjusts the flags in the commands structure and the flags in process_bands
	   to be internally consistent.

!Input Parameters:
	none
	
!Output Parameters:
	none
	
!Input/Ouput Parameters:
	commands               structure holding most of the parameters.
	process_bands          Flags for processing each band (LUNs 209900 - 2099015).

!Returns:
        nothing

!Revision History:

Revision 1.0, J. Ray, June, 2001
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-96062

!References and Credits:
  
    MODIS Science Team Members:
			
	Eric F Vermote
	Univerisity of MD.
   	NASA's GSFC Code 923
   	Greenbelt, MD , 20771
	eric@kratmos.gsfc.nasa.gov
	301-614-5521

    Developers:
      Jim Ray
      MODIS Land Science Team           Science Systems and Applications Inc.
      jim@cratmos.gsfc.nasa.gov         NASA's GSFC Code 923 
      phone (301) 614-6684              Greenbelt, MD 20771
 

!Design Notes

        Pulled out of main() to make main() more readable.
      
!END***************************************************************************
*/
{  
int bands_selected = 0;
int ich;

  if(commands->atool){
    process_bands[BAND1_1KM]=1;
    process_bands[BAND3_1KM]=1;
  } 

  for(ich=0;ich<BANDPROCESS;ich++)
    if(process_bands[ich]){
      bands_selected++;
      
    }

  if(!bands_selected){
  /* set flag to not produce reflectance product if no bands are selected.
     if not producing reflectance product, dont produce coarse product */
    commands->ref_product=0;
    commands->brdf_mode=0;
    commands->adja_on=0;
  }

  /* if no lambertian correction, don't perform any of the other operations */
  if(!commands->ref_lam){
    commands->brdf_mode=0;
    commands->adja_on=0;
  }


  /* we need certain bands for the internal reflectance cloud mask */
  if(commands->ref_product){
    if(!process_bands[BAND1_1KM]){
      process_bands[BAND1_1KM]=1;
    }
    if(!process_bands[BAND2_1KM]){
      process_bands[BAND2_1KM]=1;
    }
    
  }/* if(commands->ref_product... */

}  


